#!/usr/bin/env bash

_iocsh_bash_completion() 
{
    local cur prev opts mods
    COMPREPLY=()
    cur="${COMP_WORDS[COMP_CWORD]}"
    prev="${COMP_WORDS[COMP_CWORD-1]}"

    #  -l should only autocomplete to directories
    if [[ ${prev} == -l ]]; then
        COMPREPLY=($(compgen -d -- ${cur}))
        return 0
    fi

    #  -e should only autcomplete to filenames
    if [[ ${prev} == -e ]]; then
        COMPREPLY=($(compgen -f -- ${cur}))
        return 0
    fi

    #  -r should try to determine modules, maybe?
    # Note that this does not return anything in cellMods; but that may be a bit tough, since that will depend
    if [[ ${prev} == -r ]]; then
        if [ -n "${EPICS_BASE}" ] && [ -n "${E3_REQUIRE_VERSION}" ]; then
            mods=$(ls "${EPICS_BASE}/require/${E3_REQUIRE_VERSION}/siteMods/" 2>/dev/null)
            COMPREPLY=($(compgen -W "$mods" -- ${cur}))
        fi
        return 0
    fi

    #  -c should provide a list of reasonable commands.
    # This is imperfect, because most of these should look like:
    #    iocsh.bash -c 'var requireDebug 1'
    # i.e. with quotes. But whatever.
    if [[ ${prev} == -c ]]; then
        opts="dbLoadDatabase dbLoadRecords dbLoadTemplate epicsEnvSet epicsEnvUnset iocshLoad require runScript system var"
        COMPREPLY=($(compgen -W "$opts" -- ${cur}))
        return 0
    fi

    opts="-h -v -c -s -r -e -dg -dv -l -n"
    if [[ ${cur} == -* ]] ; then
        COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
        return 0
    fi

    COMPREPLY=($(compgen -f -- ${cur}))
}

complete -o filenames -o nospace -o bashdefault -F _iocsh_bash_completion iocsh.bash
